package org.ksv.plugin;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import com.openedge.pdt.core.template.ITemplateArgs;
import com.openedge.pdt.core.template.ITemplateGenerator;
import com.openedge.pdt.core.template.TemplateGenerationException;
import com.openedge.pdt.text.OEProject;

/**
 * New KsvProcedure Template generator. 
 * It allows to use special file as a project template file 
 * @author ksv
 *
 */
public class KsvProcedure implements ITemplateGenerator {
	
	/**
	 * Gets NL char and saves it
	 * @param lineSeparator
	 * @return
	 */
	public static synchronized KsvProcedure create(String lineSeparator) {
        nl = lineSeparator;
        KsvProcedure result = new KsvProcedure();
        return result;
    }


	/**
	 * Generates content of a new file
	 */
	public String generate( ITemplateArgs arg )
			throws TemplateGenerationException {
		// Get the current OE project
		OEProject project = KsvUtils.getActiveOEProject();		
		File templateFile = new File( templateFilename );
		
		// Look for the template file using project's propath
		if ( project != null ) {
			templateFile = KsvUtils.lookForFileInPropath( project, templateFilename );
		}
		
		// If template file is found, generate content using it
		if ( templateFile.exists() ) {
			FileReader reader = null;
			BufferedReader breader = null;
			try { 
				reader = new FileReader( templateFile );
				breader = new BufferedReader( reader );
				StringBuffer stringBuffer = new StringBuffer();
				
				// Read the alias file string by string 
				while( breader.ready() ) {					
					String line = breader.readLine();					
					stringBuffer.append( line ).append( nl );					
				}				
				return parseTags( stringBuffer.toString(), arg );
			}
			catch ( Exception e ) {
				throw new TemplateGenerationException( e );				
			}
			finally {
				try {					
					if ( breader != null ) breader.close();
					if ( reader != null ) reader.close();
				}
				catch ( Exception e ) {	
					throw new TemplateGenerationException( e );
				}
			}	
		}
		
		throw new TemplateGenerationException( "Template file " + templateFilename + " hasn't been found." );
	}
	
	/**
	 * Parses tags in the template file
	 * @param string
	 * @param arg
	 * @return 
	 */
	private static String parseTags( String string, ITemplateArgs arg ) {
		Date now = new Date();
		
		String line = string.replace( "<name>" , arg.get( "%%NewFile_Filename%%" ) );
		line = line.replace( "<date>" , DATE_TIME_FORMAT.format( now ) );
		line = line.replace( "<year>" , YEAR_FORMAT.format( now ) );
		line = line.replace( "<who>" , arg.get( "%%NewFile_Author%%" ) );
		line = line.replace( "<comment>" , arg.get( "%%NewFile_Description%%" ) );
		return line;
	}

	static String nl;
	static final String templateFilename = ".ksvprocedure";  
	private static final DateFormat DATE_TIME_FORMAT = new SimpleDateFormat("dd.MM.yyyy HH:mm"); 
	private static final DateFormat YEAR_FORMAT = new SimpleDateFormat("yyyy");

}
